from typing import Type
from celery import Celery
from celery.backends.redis import RedisBackend

from pytest import mark
import pytest


from celery_redis_poll_backend import (
    PollingRedisBackend,
    PollingRedisClusterBackend,
)


def install_redis_poll_backend():
    """Explicitly install.  Useful for local unit tests."""
    from celery.app.backends import BACKEND_ALIASES

    BACKEND_ALIASES["redis"] = "celery_redis_poll.backend:choose_redis_backend"
    BACKEND_ALIASES["rediss"] = "celery_redis_poll.backend:choose_rediss_backend"


@mark.parametrize(
    "protocol, expected_backend",
    [
        ("redis", RedisBackend),
        ("rediss", RedisBackend),
        ("redis+poll", PollingRedisBackend),
        ("rediss+poll", PollingRedisBackend),
        ("redis+cluster_poll", PollingRedisClusterBackend),
        ("rediss+cluster_poll", PollingRedisClusterBackend),
    ],
)
def test_redis_cluster_backend_installation(
    protocol: str,
    expected_backend: Type,
):
    install_redis_poll_backend()

    # Initialize Celery app with RedisClusterBackend
    options = "?ssl_cert_reqs=CERT_NONE" if "rediss" in protocol else ""
    app = Celery(
        "test_app",
        broker="redis://localhost:6379/0",
        backend=f"{protocol}://localhost:6379{options}",
    )

    # Check if the backend is set correctly
    assert isinstance(app.backend, expected_backend), (
        f"Backend is not set to {expected_backend}"
    )


if __name__ == "__main__":
    pytest.main()



from celery.backends.redis import RedisBackend
from typing import Any
import celery
from celery_redis_cluster_backend import RedisClusterBackend


def choose_backend(scheme, *args, **kwargs):
    url = kwargs.get("url", "")

    def patch_url(additional):
        nonlocal url
        kwargs["url"] = url.replace(additional, scheme, 1)

    if url.startswith("poll"):
        patch_url("poll")
        return PollingRedisBackend(*args, **kwargs)
    elif url.startswith("cluster_poll"):
        patch_url("cluster_poll")
        return PollingRedisClusterBackend(*args, **kwargs)
    else:
        return RedisBackend(*args, **kwargs)


def choose_redis_backend(*args, **kwargs):
    return choose_backend("redis", *args, **kwargs)


def choose_rediss_backend(*args, **kwargs):
    return choose_backend("rediss", *args, **kwargs)


class PollingRedisBackend(celery.backends.base.SyncBackendMixin, RedisBackend):
    """
    Disables pub/sub for getting task results and instead uses polling.
    """

    def _set(self, key: str, value: str) -> None:
        """
        Simply set value in Redis, do not publish.
        :param key:
        :param value:
        :return:
        """
        if self.expires:
            self.client.setex(key, self.expires, value)
        else:
            self.client.set(key, value)

    def on_task_call(self, *args: Any, **kwargs: Any) -> None:
        pass


class PollingRedisClusterBackend(PollingRedisBackend, RedisClusterBackend):
    """
    Same as PollingRedisBackend but with ReidsCluster for client.
    """

    pass
